'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var gitFiles = require('git-files');
var minimatch = require('minimatch');

/**
 * Returns all git files that match the given types, e.g. 'staged', 'modified'
 * in the form of a map -- fileType: fileName.
 * @param {array} fileTypes - The types of gitFiles to look for, e.g. 'staged'.
 * @param {string} pathType - The type of path for files (relative, absolute, etc.).
 * @return {object}
 */
var getFilesOfTypes = function getFilesOfTypes(fileTypes) {
	var pathType = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'relative';

	var fileMap = {};

	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = fileTypes[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			var _fileMap$fileType;

			var fileType = _step.value;

			fileMap[fileType] = fileMap[fileType] || [];
			(_fileMap$fileType = fileMap[fileType]).push.apply(_fileMap$fileType, _toConsumableArray(gitFiles[fileType](pathType)));
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}

	return fileMap;
};

/**
 * Similar to getFilesOfTypes, but returns a unique array of files.
 * @param {array} fileTypes - The types of gitFiles to look for, e.g. 'staged'.
 * @param {string} pathType - The type of path for files (relative, absolute, etc.).
 * @return {array}
 */
var getUniqueFilesOfTypes = function getUniqueFilesOfTypes(fileTypes) {
	var pathType = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'relative';

	var fileMap = getFilesOfTypes(fileTypes, pathType);
	var files = [];

	var _iteratorNormalCompletion2 = true;
	var _didIteratorError2 = false;
	var _iteratorError2 = undefined;

	try {
		for (var _iterator2 = Object.keys(fileMap).map(function (key) {
			return fileMap[key];
		})[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
			var fileArray = _step2.value;
			var _iteratorNormalCompletion3 = true;
			var _didIteratorError3 = false;
			var _iteratorError3 = undefined;

			try {
				for (var _iterator3 = fileArray[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
					var file = _step3.value;

					if (!files.includes(file)) {
						files.push(file);
					}
				}
			} catch (err) {
				_didIteratorError3 = true;
				_iteratorError3 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion3 && _iterator3.return) {
						_iterator3.return();
					}
				} finally {
					if (_didIteratorError3) {
						throw _iteratorError3;
					}
				}
			}
		}
	} catch (err) {
		_didIteratorError2 = true;
		_iteratorError2 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion2 && _iterator2.return) {
				_iterator2.return();
			}
		} finally {
			if (_didIteratorError2) {
				throw _iteratorError2;
			}
		}
	}

	return files;
};

/**
 * @param {array} files - The list of files to test.
 * @param {array} globs - The list of globs to test.
 * @param {object} [options] - Options.
 * @return {array}
 */
var matchGlobsAgainstFiles = function matchGlobsAgainstFiles(files, globs) {
	var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

	var matches = {};

	var _iteratorNormalCompletion4 = true;
	var _didIteratorError4 = false;
	var _iteratorError4 = undefined;

	try {
		for (var _iterator4 = files[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
			var file = _step4.value;
			var _iteratorNormalCompletion5 = true;
			var _didIteratorError5 = false;
			var _iteratorError5 = undefined;

			try {
				for (var _iterator5 = globs[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
					var glob = _step5.value;

					if (fileMatchesGlob(file, glob, options)) {
						matches[file] = true;
					}
				}
			} catch (err) {
				_didIteratorError5 = true;
				_iteratorError5 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion5 && _iterator5.return) {
						_iterator5.return();
					}
				} finally {
					if (_didIteratorError5) {
						throw _iteratorError5;
					}
				}
			}
		}
	} catch (err) {
		_didIteratorError4 = true;
		_iteratorError4 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion4 && _iterator4.return) {
				_iterator4.return();
			}
		} finally {
			if (_didIteratorError4) {
				throw _iteratorError4;
			}
		}
	}

	return Object.keys(matches);
};

function fileMatchesGlob(file, glob) {
	var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

	// for minimatch
	options.nocase = !options.caseSensitive;

	var globsToTry = options.strict ? [glob] : ['*' + glob + '*', '**/*' + glob + '*', '**/*' + glob + '*/**/*', '**' + glob + '*/**/*', '**/*' + glob + '**/*'];

	var _iteratorNormalCompletion6 = true;
	var _didIteratorError6 = false;
	var _iteratorError6 = undefined;

	try {
		for (var _iterator6 = globsToTry[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
			var globTry = _step6.value;

			if (minimatch(file, globTry, options)) {
				return true;
			}
		}
	} catch (err) {
		_didIteratorError6 = true;
		_iteratorError6 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion6 && _iterator6.return) {
				_iterator6.return();
			}
		} finally {
			if (_didIteratorError6) {
				throw _iteratorError6;
			}
		}
	}

	return false;
}

module.exports = {
	getFilesOfTypes: getFilesOfTypes,
	getUniqueFilesOfTypes: getUniqueFilesOfTypes,
	matchGlobsAgainstFiles: matchGlobsAgainstFiles
};